@@ -244,7 +244,9 @@ int irc_channel_add_user(irc_channel_t *ic, irc_user_t *iu)
 
 	ic->users = g_slist_insert_sorted(ic->users, icu, irc_channel_user_cmp);
 
-	irc_channel_update_ops(ic, set_getstr(&ic->irc->b->set, "ops"));
+	if (iu == ic->irc->user || iu == ic->irc->root) {
+		irc_channel_update_ops(ic, set_getstr(&ic->irc->b->set, "ops"));
+	}
 
 	if (iu == ic->irc->user || ic->flags & IRC_CHANNEL_JOINED) {
 		ic->flags |= IRC_CHANNEL_JOINED;
@@ -635,7 +637,7 @@ char *irc_channel_name_gen(irc_t *irc, const char *hint)
 
 	irc_channel_name_strip(name);
 
-	if (set_getbool(&irc->b->set, "lcnicks")) {
+	if (set_getbool(&irc->b->set, "nick_lowercase")) {
 		nick_lc(irc, name + 1);
 	}
 
