@@ -66,8 +66,9 @@ void ssl_init(void)
 
 	SSL_library_init();
 
-	meth = TLSv1_client_method();
+	meth = SSLv23_client_method();
 	ssl_ctx = SSL_CTX_new(meth);
+	SSL_CTX_set_options(ssl_ctx, SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3);
 
 	initialized = TRUE;
 }
