@@ -27,7 +27,8 @@
 static xt_status jabber_chat_join_failed(struct im_connection *ic, struct xt_node *node, struct xt_node *orig);
 static xt_status jabber_chat_self_message(struct im_connection *ic, struct xt_node *node, struct xt_node *orig);
 
-struct groupchat *jabber_chat_join(struct im_connection *ic, const char *room, const char *nick, const char *password)
+struct groupchat *jabber_chat_join(struct im_connection *ic, const char *room, const char *nick, const char *password,
+                                   gboolean always_use_nicks)
 {
 	struct jabber_chat *jc;
 	struct xt_node *node;
@@ -58,6 +59,10 @@ struct groupchat *jabber_chat_join(struct im_connection *ic, const char *room, c
 		return NULL;
 	}
 
+	if (always_use_nicks) {
+		jc->flags = JCFLAG_ALWAYS_USE_NICKS;
+	}
+
 	/* roomjid isn't normalized yet, and we need an original version
 	   of the nick to send a proper presence update. */
 	jc->my_full_jid = roomjid;
@@ -94,7 +99,7 @@ struct groupchat *jabber_chat_with(struct im_connection *ic, char *who)
 	g_free(uuid);
 	g_free(cserv);
 
-	c = jabber_chat_join(ic, rjid, jd->username, NULL);
+	c = jabber_chat_join(ic, rjid, jd->username, NULL, FALSE);
 	g_free(rjid);
 	if (c == NULL) {
 		return NULL;
@@ -161,6 +166,7 @@ void jabber_chat_free(struct groupchat *c)
 
 	jabber_buddy_remove_bare(c->ic, jc->name);
 
+	g_free(jc->last_sent_message);
 	g_free(jc->my_full_jid);
 	g_free(jc->name);
 	g_free(jc->invite);
@@ -182,6 +188,9 @@ int jabber_chat_msg(struct groupchat *c, char *message, int flags)
 
 	jabber_cache_add(ic, node, jabber_chat_self_message);
 
+	g_free(jc->last_sent_message);
+	jc->last_sent_message = g_strdup(message);
+
 	return !jabber_write_packet(ic, node);
 }
 
@@ -243,6 +252,19 @@ void jabber_chat_invite(struct groupchat *c, char *who, char *message)
 	xt_free_node(node);
 }
 
+static int jabber_chat_has_other_resources(struct im_connection *ic, struct jabber_buddy *bud)
+{
+	struct jabber_buddy *cur;
+
+	for (cur = jabber_buddy_by_jid(ic, bud->bare_jid, GET_BUDDY_FIRST); cur; cur = cur->next) {
+		if (cur != bud && jabber_compare_jid(cur->ext_jid, bud->ext_jid)) {
+			return TRUE;
+		}
+	}
+	
+	return FALSE;
+}
+
 /* Not really the same syntax as the normal pkt_ functions, but this isn't
    called by the xmltree parser directly and this way I can add some extra
    parameters so we won't have to repeat too many things done by the caller
@@ -327,12 +349,17 @@ void jabber_chat_pkt_presence(struct im_connection *ic, struct jabber_buddy *bud
 		if (s) {
 			*s = 0; /* Should NEVER be NULL, but who knows... */
 		}
+
+		if (bud != jc->me && (jc->flags & JCFLAG_ALWAYS_USE_NICKS) && !(bud->flags & JBFLAG_IS_ANONYMOUS)) {
+			imcb_buddy_nick_change(ic, bud->ext_jid, bud->resource);
+		}
+
 		imcb_chat_add_buddy(chat, bud->ext_jid);
 		if (s) {
 			*s = '/';
 		}
 	} else if (type) { /* type can only be NULL or "unavailable" in this function */
-		if ((bud->flags & JBFLAG_IS_CHATROOM) && bud->ext_jid) {
+		if ((bud->flags & JBFLAG_IS_CHATROOM) && bud->ext_jid && !jabber_chat_has_other_resources(ic, bud)) {
 			char *reason = NULL;
 			char *status = NULL;
 			char *status_text = NULL;
@@ -442,7 +469,8 @@ void jabber_chat_pkt_message(struct im_connection *ic, struct jabber_buddy *bud,
 	}
 
 	if (subject && chat) {
-		char *subject_text = subject->text_len > 0 ? subject->text : "";
+		char empty[1] = "";
+		char *subject_text = subject->text_len > 0 ? subject->text : empty;
 		if (g_strcmp0(chat->topic, subject_text) != 0) {
 			bare_jid = (bud) ? jabber_get_bare_jid(bud->ext_jid) : NULL;
 			imcb_chat_topic(chat, bare_jid, subject_text,
@@ -469,10 +497,16 @@ void jabber_chat_pkt_message(struct im_connection *ic, struct jabber_buddy *bud,
 	} else if (chat != NULL && bud == NULL && nick == NULL) {
 		imcb_chat_log(chat, "From conference server: %s", body->text);
 		return;
-	} else if (jc && jc->flags & JCFLAG_MESSAGE_SENT && bud == jc->me &&
-		   (jabber_cache_handle_packet(ic, node) == XT_ABORT)) {
-		/* Self message marked by this bitlbee, don't show it */
-		return;
+	} else if (jc && jc->flags & JCFLAG_MESSAGE_SENT && bud == jc->me) {
+		if (jabber_cache_handle_packet(ic, node) == XT_ABORT) {
+			/* Self message marked by this bitlbee, don't show it */
+			return;
+		} else if (xt_find_attr(node, "id") == NULL &&
+		           g_strcmp0(body->text, jc->last_sent_message) == 0) {
+			/* Some misbehaving servers (like slack) eat the ids and echo anyway.
+			 * Try to detect those cases by comparing against the last sent message. */
+			return;
+		}
 	}
 
 	if (bud) {
