@@ -39,24 +39,102 @@
 GSList *connections;
 
 #ifdef WITH_PLUGINS
+GList *plugins = NULL;
+
+static gint pluginscmp(gconstpointer a, gconstpointer b, gpointer data)
+{
+	const struct plugin_info *ia = a;
+	const struct plugin_info *ib = b;
+
+	return g_strcasecmp(ia->name, ib->name);
+}
+
+/* semi-private */
+gboolean plugin_info_validate(struct plugin_info *info, const char *path)
+{
+	GList *l;
+	gboolean loaded = FALSE;
+
+	if (!path) {
+		path = "(null)";
+	}
+
+	if (info->abiver != BITLBEE_ABI_VERSION_CODE) {
+		log_message(LOGLVL_ERROR,
+			    "`%s' uses ABI %u but %u is required\n",
+			    path, info->abiver,
+			    BITLBEE_ABI_VERSION_CODE);
+		return FALSE;
+	}
+
+	if (!info->name || !info->version) {
+		log_message(LOGLVL_ERROR,
+			    "Name or version missing from the "
+			    "plugin info in `%s'\n", path);
+		return FALSE;
+	}
+
+	for (l = plugins; l; l = l->next) {
+		struct plugin_info *i = l->data;
+
+		if (g_strcasecmp(i->name, info->name) == 0) {
+			loaded = TRUE;
+			break;
+		}
+	}
+
+	if (loaded) {
+		log_message(LOGLVL_WARNING,
+			    "%s plugin already loaded\n",
+			    info->name);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+/* semi-private */
+gboolean plugin_info_add(struct plugin_info *info)
+{
+	plugins = g_list_insert_sorted_with_data(plugins, info, pluginscmp, NULL);
+	return TRUE;
+}
+
 gboolean load_plugin(char *path)
 {
+	struct plugin_info *info = NULL;
+	struct plugin_info * (*info_function) (void) = NULL;
 	void (*init_function) (void);
 
 	GModule *mod = g_module_open(path, G_MODULE_BIND_LAZY);
 
 	if (!mod) {
-		log_message(LOGLVL_ERROR, "Can't find `%s', not loading (%s)\n", path, g_module_error());
+		log_message(LOGLVL_ERROR, "Error loading plugin `%s': %s\n", path, g_module_error());
 		return FALSE;
 	}
 
+	if (g_module_symbol(mod, "init_plugin_info", (gpointer *) &info_function)) {
+		info = info_function();
+
+		if (!plugin_info_validate(info, path)) {
+			g_module_close(mod);
+			return FALSE;
+		}
+	} else {
+		log_message(LOGLVL_WARNING, "Can't find function `init_plugin_info' in `%s'\n", path);
+	}
+
 	if (!g_module_symbol(mod, "init_plugin", (gpointer *) &init_function)) {
 		log_message(LOGLVL_WARNING, "Can't find function `init_plugin' in `%s'\n", path);
+		g_module_close(mod);
 		return FALSE;
 	}
 
-	init_function();
+	if (info_function) {
+		plugin_info_add(info);
+	}
 
+	init_function();
 	return TRUE;
 }
 
@@ -72,6 +150,10 @@ void load_plugins(void)
 		char *path;
 
 		while ((entry = g_dir_read_name(dir))) {
+			if (!g_str_has_suffix(entry, "." G_MODULE_SUFFIX)) {
+				continue;
+			}
+
 			path = g_build_filename(global.conf->plugindir, entry, NULL);
 			if (!path) {
 				log_message(LOGLVL_WARNING, "Can't build path for %s\n", entry);
@@ -86,6 +168,11 @@ void load_plugins(void)
 		g_dir_close(dir);
 	}
 }
+
+GList *get_plugins()
+{
+	return plugins;
+}
 #endif
 
 GList *protocols = NULL;
@@ -126,11 +213,48 @@ gboolean is_protocol_disabled(const char *name)
 	return g_list_find_custom(disabled_protocols, name, proto_name_cmp) != NULL;
 }
 
+/* Returns heap allocated string with text attempting to explain why a protocol is missing 
+ * Free the return value with g_free() */
+char *explain_unknown_protocol(const char *name)
+{
+	char *extramsg = NULL;
+
+	if (is_protocol_disabled(name)) {
+		return g_strdup("Protocol disabled in the global config (bitlbee.conf)");
+	}
+
+	if (strcmp(name, "yahoo") == 0) {
+		return g_strdup("The old yahoo protocol is gone, try the funyahoo++ libpurple plugin instead.");
+	}
+
+#ifdef WITH_PURPLE
+	if ((strcmp(name, "msn") == 0) ||
+	    (strcmp(name, "loubserp-mxit") == 0) ||
+	    (strcmp(name, "myspace") == 0)) {
+		return g_strdup("This protocol has been removed from your libpurple version.");
+	}
+
+	if (strcmp(name, "hipchat") == 0) {
+		return g_strdup("This account type isn't supported by libpurple's jabber.");
+	}
+
+#else
+	if (strcmp(name, "aim") == 0 || strcmp(name, "icq") == 0) {
+		return g_strdup("This account uses libpurple specific aliases for oscar. "
+		                "Re-add the account with `account add oscar ...'");
+	}
+
+	extramsg = "If this is a libpurple plugin, you might need to install bitlbee-libpurple instead.";
+#endif
+	return g_strconcat("The protocol plugin is not installed or could not be loaded. "
+	                   "Use the `plugins' command to list available protocols. ",
+	                   extramsg, NULL);
+}
+
 void nogaim_init()
 {
 	extern void msn_initmodule();
 	extern void oscar_initmodule();
-	extern void byahoo_initmodule();
 	extern void jabber_initmodule();
 	extern void twitter_initmodule();
 	extern void purple_initmodule();
@@ -143,10 +267,6 @@ void nogaim_init()
 	oscar_initmodule();
 #endif
 
-#ifdef WITH_YAHOO
-	byahoo_initmodule();
-#endif
-
 #ifdef WITH_JABBER
 	jabber_initmodule();
 #endif
@@ -164,6 +284,16 @@ void nogaim_init()
 #endif
 }
 
+GList *get_protocols()
+{
+	return protocols;
+}
+
+GList *get_protocols_disabled()
+{
+	return disabled_protocols;
+}
+
 GSList *get_connections()
 {
 	return connections;
@@ -478,9 +608,8 @@ void imcb_remove_buddy(struct im_connection *ic, const char *handle, char *group
 	bee_user_free(ic->bee, bee_user_by_handle(ic->bee, ic, handle));
 }
 
-/* Mainly meant for ICQ (and now also for Jabber conferences) to allow IM
-   modules to suggest a nickname for a handle. */
-void imcb_buddy_nick_hint(struct im_connection *ic, const char *handle, const char *nick)
+/* Implements either imcb_buddy_nick_hint() or imcb_buddy_nick_change() depending on the value of 'change' */
+static void buddy_nick_hint_or_change(struct im_connection *ic, const char *handle, const char *nick, gboolean change)
 {
 	bee_t *bee = ic->bee;
 	bee_user_t *bu = bee_user_by_handle(bee, ic, handle);
@@ -492,11 +621,24 @@ void imcb_buddy_nick_hint(struct im_connection *ic, const char *handle, const ch
 	g_free(bu->nick);
 	bu->nick = g_strdup(nick);
 
-	if (bee->ui->user_nick_hint) {
+	if (change && bee->ui->user_nick_change) {
+		bee->ui->user_nick_change(bee, bu, nick);
+	} else if (!change && bee->ui->user_nick_hint) {
 		bee->ui->user_nick_hint(bee, bu, nick);
 	}
 }
 
+/* Soft variant, for newly created users. Does nothing if it's already online */
+void imcb_buddy_nick_hint(struct im_connection *ic, const char *handle, const char *nick)
+{
+	buddy_nick_hint_or_change(ic, handle, nick, FALSE);
+}
+
+/* Hard variant, always changes the nick */
+void imcb_buddy_nick_change(struct im_connection *ic, const char *handle, const char *nick)
+{
+	buddy_nick_hint_or_change(ic, handle, nick, TRUE);
+}
 
 struct imcb_ask_cb_data {
 	struct im_connection *ic;
@@ -620,6 +762,9 @@ int imc_away_send_update(struct im_connection *ic)
 	       : set_getstr(&ic->bee->set, "away");
 	if (away && *away) {
 		GList *m = ic->acc->prpl->away_states(ic);
+		if (m == NULL) {
+			return 0;
+		}
 		msg = ic->acc->flags & ACC_FLAG_AWAY_MESSAGE ? away : NULL;
 		away = imc_away_state_find(m, away, &msg) ? :
 		       (imc_away_state_find(m, "away", &msg) ? : m->data);
