@@ -344,6 +344,8 @@ void twitter_login_finish(struct im_connection *ic)
 	           !(td->flags & TWITTER_HAVE_FRIENDS)) {
 		imcb_log(ic, "Getting contact list");
 		twitter_get_friends_ids(ic, -1);
+		twitter_get_mutes_ids(ic, -1);
+		twitter_get_noretweets_ids(ic, -1);
 	} else {
 		twitter_main_loop_start(ic);
 	}
@@ -684,6 +686,12 @@ static void twitter_logout(struct im_connection *ic)
 			b_event_remove(td->filter_update_id);
 		}
 
+		g_slist_foreach(td->mutes_ids, (GFunc) g_free, NULL);
+		g_slist_free(td->mutes_ids);
+
+		g_slist_foreach(td->noretweets_ids, (GFunc) g_free, NULL);
+		g_slist_free(td->noretweets_ids);
+
 		http_close(td->stream);
 		twitter_filter_remove_all(ic);
 		oauth_info_free(td->oauth_info);
@@ -942,7 +950,8 @@ static void twitter_handle_command(struct im_connection *ic, char *message)
 		goto eof;
 	} else if ((g_strcasecmp(cmd[0], "favourite") == 0 ||
 	            g_strcasecmp(cmd[0], "favorite") == 0 ||
-	            g_strcasecmp(cmd[0], "fav") == 0) && cmd[1]) {
+	            g_strcasecmp(cmd[0], "fav") == 0 ||
+	            g_strcasecmp(cmd[0], "like") == 0) && cmd[1]) {
 		if ((id = twitter_message_id_from_command_arg(ic, cmd[1], NULL))) {
 			twitter_favourite_tweet(ic, id);
 		} else {
@@ -955,6 +964,12 @@ static void twitter_handle_command(struct im_connection *ic, char *message)
 	} else if (g_strcasecmp(cmd[0], "unfollow") == 0 && cmd[1]) {
 		twitter_remove_buddy(ic, cmd[1], NULL);
 		goto eof;
+	} else if (g_strcasecmp(cmd[0], "mute") == 0 && cmd[1]) {
+		twitter_mute_create_destroy(ic, cmd[1], 1);
+		goto eof;
+	} else if (g_strcasecmp(cmd[0], "unmute") == 0 && cmd[1]) {
+		twitter_mute_create_destroy(ic, cmd[1], 0);
+		goto eof;
 	} else if ((g_strcasecmp(cmd[0], "report") == 0 ||
 	            g_strcasecmp(cmd[0], "spam") == 0) && cmd[1]) {
 		char *screen_name;
@@ -1076,7 +1091,7 @@ void twitter_initmodule()
 {
 	struct prpl *ret = g_new0(struct prpl, 1);
 
-	ret->options = OPT_NOOTR;
+	ret->options = PRPL_OPT_NOOTR | PRPL_OPT_NO_PASSWORD;
 	ret->name = "twitter";
 	ret->login = twitter_login;
 	ret->init = twitter_init;
@@ -1103,5 +1118,6 @@ void twitter_initmodule()
 	/* And an identi.ca variant: */
 	ret = g_memdup(ret, sizeof(struct prpl));
 	ret->name = "identica";
+	ret->options =  PRPL_OPT_NOOTR;
 	register_protocol(ret);
 }
