@@ -137,6 +137,11 @@ struct {
 		5,
 		"aaaaa\naaaaa\naaaaa\naaaaa\naaaaa\naaaaa\naaaaa\na",
 	},
+	{
+		"áááááááááá",
+		11,
+		"ááááá\nááááá",
+	},
 	{
 		NULL
 	}
