@@ -79,6 +79,14 @@ int main(int argc, char *argv[])
 		return(1);
 	}
 
+	if (global.conf->runmode == RUNMODE_INETD) {
+		log_link(LOGLVL_ERROR, LOGOUTPUT_IRC);
+		log_link(LOGLVL_WARNING, LOGOUTPUT_IRC);
+	} else {
+		log_link(LOGLVL_ERROR, LOGOUTPUT_CONSOLE);
+		log_link(LOGLVL_WARNING, LOGOUTPUT_CONSOLE);
+	}
+
 	b_main_init();
 
 	/* libpurple doesn't like fork()s after initializing itself, so if
@@ -103,23 +111,20 @@ int main(int argc, char *argv[])
 		return(1);
 	}
 
-	if (global.conf->runmode == RUNMODE_INETD) {
-		log_link(LOGLVL_ERROR, LOGOUTPUT_IRC);
-		log_link(LOGLVL_WARNING, LOGOUTPUT_IRC);
+	global.auth = auth_init(global.conf->auth_backend);
+	if (global.conf->auth_backend && global.auth == NULL) {
+		log_message(LOGLVL_ERROR, "Unable to load authentication backend '%s'", global.conf->auth_backend);
+		return(1);
+	}
 
+	if (global.conf->runmode == RUNMODE_INETD) {
 		i = bitlbee_inetd_init();
 		log_message(LOGLVL_INFO, "%s %s starting in inetd mode.", PACKAGE, BITLBEE_VERSION);
 
 	} else if (global.conf->runmode == RUNMODE_DAEMON) {
-		log_link(LOGLVL_ERROR, LOGOUTPUT_CONSOLE);
-		log_link(LOGLVL_WARNING, LOGOUTPUT_CONSOLE);
-
 		i = bitlbee_daemon_init();
 		log_message(LOGLVL_INFO, "%s %s starting in daemon mode.", PACKAGE, BITLBEE_VERSION);
 	} else if (global.conf->runmode == RUNMODE_FORKDAEMON) {
-		log_link(LOGLVL_ERROR, LOGOUTPUT_CONSOLE);
-		log_link(LOGLVL_WARNING, LOGOUTPUT_CONSOLE);
-
 		/* In case the operator requests a restart, we need this. */
 		old_cwd = g_malloc(256);
 		if (getcwd(old_cwd, 255) == NULL) {
@@ -141,12 +146,17 @@ int main(int argc, char *argv[])
 	    (!getuid() || !geteuid())) {
 		struct passwd *pw = NULL;
 		pw = getpwnam(global.conf->user);
-		if (pw) {
-			initgroups(global.conf->user, pw->pw_gid);
-			setgid(pw->pw_gid);
-			setuid(pw->pw_uid);
-		} else {
-			log_message(LOGLVL_WARNING, "Failed to look up user %s.", global.conf->user);
+		if (!pw) {
+			log_message(LOGLVL_ERROR, "Failed to look up user %s.", global.conf->user);
+
+		} else if (initgroups(global.conf->user, pw->pw_gid) != 0) {
+			log_message(LOGLVL_ERROR, "initgroups: %s.", strerror(errno));
+
+		} else if (setgid(pw->pw_gid) != 0) {
+			log_message(LOGLVL_ERROR, "setgid(%d): %s.", pw->pw_gid, strerror(errno));
+
+		} else if (setuid(pw->pw_uid) != 0) {
+			log_message(LOGLVL_ERROR, "setuid(%d): %s.", pw->pw_uid, strerror(errno));
 		}
 	}
 
@@ -275,9 +285,10 @@ void sighandler_shutdown_setup()
 /* Signal handler for SIGTERM and SIGINT */
 static void sighandler_shutdown(int signal)
 {
+	int unused G_GNUC_UNUSED;
 	/* Write a single null byte to the pipe, just to send a message to the main loop.
 	 * This gets handled by bitlbee_shutdown (the b_input_add callback for this pipe) */
-	write(shutdown_pipe.fd[1], "", 1);
+	unused = write(shutdown_pipe.fd[1], "", 1);
 }
 
 /* Signal handler for SIGSEGV
@@ -286,13 +297,14 @@ static void sighandler_shutdown(int signal)
 static void sighandler_crash(int signal)
 {
 	GSList *l;
+	int unused G_GNUC_UNUSED;
 	const char *message = "ERROR :BitlBee crashed! (SIGSEGV received)\r\n";
 	int len = strlen(message);
 
 	for (l = irc_connection_list; l; l = l->next) {
 		irc_t *irc = l->data;
 		sock_make_blocking(irc->fd);
-		write(irc->fd, message, len);
+		unused = write(irc->fd, message, len);
 	}
 
 	raise(signal);
